// ===============================================================
// udemo.DemoInterface: NATIVE CLASS used to control the demo playback driver!
// This provides for a cleaner method for control rather than standard console commands.
// note: driver is reponsible for checks including server changing of timedilation and pauser!
// ===============================================================

class DemoInterface expands Object
native;

//variables:
var const PlayerPawn DemoSpec;  //the spectator subclass
var const uDemoDriver DemoDriver; //the demo driver. only valid in cpp
//useful in vars to read in script:
var const float mySpeed; //current speed of demo
var const byte PlayBackMode; //current playback mode
var const bool bDoingMessagePlay; //if true, no rendering is taking place.  This is only for message grabbing!

//native functions:   (add interval subdivion ammount?)
native final function SetSpeed (float newSpeed); //playback speed control
native final function JumpBack (); //jumps all the way back to beginning (at starttime). deletes mostly all non-flagged actors
native final function ReadTo (float time); //read ahead until have reached time.
native final function ReadCache (float ToTime,float inc); //read until reached toTime + tick non-pawn actors by inc
native final function float GetCurrentTime(); //This is the current time (-start) demo is on
native final function float GetTotalTime(); //time demo lasts (1.0 speed) (start-end)
native final function int GetCurrentFrame(); //return current frame number (-start frame)
native final function int GetTotalFrames(); //return the number of frames in the demo that is playing
native final function PauseDemo(bool bPause); //safely controls demo pausing.
native final function byte IsPaused(); //0=no, 1=paused in demo (ex. admin hit pause), 2=user requested
native final function SetPlayBackMode(byte a); //playback modes: 0=timebased, 1=framebased, 2=nocap
native final function float GetStartTime(); //returns the time demo was at when starting play (After player spawned)

//non-native functions:
function GotoFrame (float Time){ //controls jumping
   if (Time > GetTotalTime() || Time < GetStartTime())  //invalid
      return;
   if (abs(Time-GetCurrentTime()) < 0.5) //too small interval.. don't jump!
      return;
   if (Time > GetCurrentTime() && Time < GetCurrentTime() + fmax(2*class'DemoSettings'.default.CacheSeconds,5.0))
      ReadCache (Time+GetStartTime(),class'demosettings'.default.TickSize); //more reliable to keep reading w/ cache
   else{ //must do complex jumping method...
      Time+=GetStartTime();
      if (Time<GetCurrentTime())
        JumpBack(); //go backwards in demo
      if (time-class'DemoSettings'.default.CacheSeconds>GetCurrentTime())
        ReadTo(time-class'DemoSettings'.default.CacheSeconds); //read forwards
      ReadCache(Time,class'DemoSettings'.default.TickSize); //now do caching
   }
}


//events:

//in clientdemos, this is the playerpawn the spectator should "link to" for information (HUD, etc.) none in serverdemo!
//passes other useful info as well ;p
event LinkToPlayer (PlayerPawn p){
  DemoPlaybackSpec(DemoSpec).Driver = self;  //give pointer to self
  log (DemoSpec@"(view port of '"$DemoSpec.Player$"') linked to"@p,'Udemo');
  if (!bDoingMessagePlay)
    DemoSpec.spawn(class'DemoNotify');    //makes all actors past this spawn !bDifficulty3
  log ("Sucessfully Spawned notify!",'Udemo');
  //do stuff here
}

event NetPacketReceived(); //called each tick if message grabbing
defaultproperties {
}
